
class HideNPeekMHv4b extends Mutator config(HideNPeekMHv4b);

#exec TEXTURE IMPORT NAME=Pinky FILE=Textures\Pinky.bmp GROUP=Skins FLAGS=2 LODSET=2

var bool bInitialized;
var config int FixedMonsters, TimeInterval;
var config bool VisualTestingMode;

event PostBeginPlay()
{
	//Prevents multiple copies of the mod from running
	if (bInitialized)
		Return;
	bInitialized = True;
	//Sets the timer to begin the monster checks every (TimeInterval) period
	SetTimer(TimeInterval,true);
}

//Quick check to ensure that the playerpawns are relevant. Will hardly ever be needed honestly.
function ModifyPlayer(Pawn Other)
{
	if( (Other.IsA('PlayerPawn') || Other.IsA('Bot')) && Other.bIsPlayer && !Other.IsA('Spectator'))
		Other.bAlwaysRelevant=True;
		
	super.ModifyPlayer(Other);
}

//When the timer triggers it runs the monster fix code
function Timer()
{
	FixMonsters();
	Super.Timer();
}

function FixMonsters()
{
	local ScriptedPawn S;
	local Pawn P;
	local int Counter, CounterB, CounterC;
    local bool DoSecondPass, DoThirdPass;
	local int i;

	Counter = 0;
	CounterB = 0;
	CounterC = 0;
	DoSecondPass = True;
	DoThirdPass = True;

	// First pass checks for enough existing FixedMonsters. In this first pass we poll the array
	// of monsters and see if enough of them are already AlwaysRelevant. If not then it will run a second pass.
	for (P=Level.PawnList; P!=None; P=P.NextPawn)
	{
		S=ScriptedPawn(P);
		if(S!=None)
		{
			if (S.bAlwaysRelevant == True)
			{
				if(VisualTestingMode)
				{
					For (i=0; i<8; i++)
						S.MultiSkins[i] = Texture'Pinky';
				}
				Counter++;

				if (Counter >= FixedMonsters) 
				{
					DoSecondPass = False;
                    DoThirdPass = False;
					break;
				}
			}
		}
	}
	
	//This second pass will take the number of fixed monsters and attempt to fill in the remainder with monsters who
	//are in specific states. These are the monsters actively fighting at the moment and the most likely to need to
	//be fixed. If we still don't get enough then it will trigger the third pass.

	if(DoSecondPass)
	{
		for (P=Level.PawnList; P!=None; P=P.NextPawn)
		{
			S=ScriptedPawn(P);
			if(S!=None)
			{
				if ((S.bAlwaysRelevant == False) && ((S.IsInState('RangedAttack')) || (S.IsInState('Attacking')) || (S.IsInState('Charging'))))
				{
					S.bAlwaysRelevant = True;
					if(VisualTestingMode)
					{
						For (i=0; i<8; i++)
							S.MultiSkins[i] = Texture'Pinky';
					}
					CounterB++;
				    if ((CounterB + Counter) >= FixedMonsters) 
				    {
					    DoThirdPass = False;
					    break;
				    }
				}
			}
		}
	}
	
	//Third pass just grabs from the scriptedpawn list and fills out any remaining FixedMonsters that are needed
	if(DoThirdPass)
	{
		for (P=Level.PawnList; P!=None; P=P.NextPawn)
		{
			S=ScriptedPawn(P);
			if(S!=None)
			{
				if (S.bAlwaysRelevant == False)
				{
					S.bAlwaysRelevant=True;
					if(VisualTestingMode)
					{
						For (i=0; i<8; i++)
							S.MultiSkins[i] = Texture'Pinky';
					}
					CounterC++;
					if ((CounterC + CounterB + Counter) >= FixedMonsters)
						break;
				}
			}
		}
	}
}

defaultproperties
{
	FixedMonsters=50
	TimeInterval=30
	VisualTestingMode=False
}
